/*
*
*  GroupAction.java
*
*  SPECT: Simple Portal-Enhancing Configuration Tool
*  Copyright (C) 1999, 2000 Ray Ontko & Co.
*  http://www.ontko.com/
*
*  This program is a simple-to-use configuration tool for Brio.Portal
*  (formerly ReportMart).
*
*  "Brio.Portal" is a trademark of Brio Technology, http://www.brio.com/ .
* 
*
*  This program is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by 
*  the Free Software Foundation; either version 2 of the License, or 
*  (at your option) any later version. 
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of 
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
*  Public License for more details. 
* 
*  A copy of the GPL is available online the GNU web site: 
* 
*    http://www.gnu.org/copyleft/gpl.html
* 
*/ 

import java.io.*;
import java.util.*;
import com.sqribe.rm.*;

/*******************************************************************
*
*  GroupAction parses a group-related command from the control file
*  and performs the specified action on ReportMart.  The allowed actions
*  are create group, update group, and delete group.  Because of 
*  limitations of the current RM API (Group does not have an update()
*  method, and thus has no way to commit any changes to the database),
*  create group and update group are not fully functional.  Groups added
*  with the create command are owned by the default user.  Group names,
*  permissions, and owner cannot be altered using the API.  The admin 
*  tool allows a user to perform these alterations.
*
*******************************************************************/
public class GroupAction extends BaseAction {

  private Repository repository;
  private static RMHashtable default_keywords;
  private RMHashtable keywords;
  private StreamTokenizer st;
  private int action;
  private int lineno;

  // Load default values into keyword hashtable
  static {
    default_keywords = new RMHashtable();
    RMProperty gp = new RMProperty("NAME","");
    default_keywords.put(gp.name, gp);
    gp = new RMProperty("DESCRIPTION","");
    default_keywords.put(gp.name, gp);
    gp = new RMProperty("OWNER",RMLoader.username);
    default_keywords.put(gp.name, gp);
    gp = new RMProperty("PERMISSIONS","740");
    default_keywords.put(gp.name, gp);
  }

  /**
  *
  *  Constructor calls methods to parse the command from the control file,
  *  and execute the action.
  *
  **/
  public GroupAction(Repository repository, int action, StreamTokenizer st, 
         int lineno) throws IOException{
    super(action, st, lineno);
    this.repository = repository;
    this.action = action;
    this.type = "GROUP";
    keywords = (RMHashtable)default_keywords.clone();
    parse(default_keywords, keywords);
    doAction();
  }

  /** After successfully parsing a GROUP action, do the action
   ** specified with the keyword values given, or defaulted **/
  public int doAction() {
    switch (action) {
    case RMLoader.CREATE :
      createGroup();
      break;
    case RMLoader.UPDATE :
      updateGroup();
      break;
    case RMLoader.DELETE :
      deleteGroup();
    default :
      error("invalid action for GROUP, " + RMLoader.decodeAction(action));
    }
    return 0;
  }

  /**
  *
  * Adds a group to ReportMart
  *
  **/
  public void createGroup() {
    RMProperty rmp = (RMProperty)keywords.get("NAME");
    String name = rmp.value;

    // check to see if group already exists.  if it doesn't, create
    // it. if it does, check config option WHEN-PREEXISTS to see what
    // to do.
    try {
      repository.getGroup(name);
      switch (RMLoader.config.when_preexists) {
      case RMConfig.ERROR :
        error("group " + name + " already exists");
        printBadRecord();
        return;
      case RMConfig.WARNING :
        warning("group " + name + " already exists");
      case RMConfig.IGNORE :
        printGoodRecord();
        return;
      case RMConfig.UPDATE :
        updateGroup();
      }
    }  
    catch (ReportMartException rme) {} // Group doesn't exist, so we'll add it

    rmp = (RMProperty)keywords.get("PERMISSIONS");
    Permissions permissions = convertUmask(rmp.value);

    // As of 6/99, the RM API does not support the Group.update() method,
    // which would be necessary to update any group properties.  When this
    // capability is added, this code should be altered to support it.
    rmp = (RMProperty)keywords.get("OWNER");
    if (!rmp.is_default)
      warning("cannot set owner of group " + name + ", the RM API currently " +
        "does not support this operation");

    // When Group.update() is available, uncomment this code. 
    // try {
    //   User user = repository.getUser(rmp.value);
    // }
    // catch (ReportMartException rme) {
    //   error("unable to use user " + rmp.value + " as owner of group " +
    //     name + ", " + rme.getMessage());
    //   printBadMessage();
    //   return;
    // }

    try {
      Group group = repository.addGroup(name, permissions);
    }
    catch (ReportMartException rme) {
      error("unable to create group " + name + ", " + rme.getMessage());
      printBadRecord();
      return;
    }

    log("Group \"" + name + "\" successfully created");

    // When Group.update is available, uncomment this code and alter, if
    // necessary.
    // try {
    //   group.setOwner(user);
    //   group.update();
    // }
    // catch(ReportMartException rme) {
    //   error("unable to set user " + user.getName() + " as owner of group " +
    //     name + ", " + rme.getMessage();
    //   printBadRecord();
    //   return;
    // }
    // RMLoader.logMsg("User " + user.getName() + " successfully set as " +
    //   "owner of group " + name);

    printGoodRecord();
  }

  /**
  *
  *  Update a ReportMart group.  This action is not currently supported by
  *  the RM API.
  *
  **/
  private void updateGroup() {
    RMProperty rmp = (RMProperty)keywords.get("NAME");
    String name = rmp.value;
    try {
      Group group = repository.getGroup(name);
      rmp = (RMProperty)keywords.get("DESCRIPTION");
      if (!rmp.is_default) {  
        // no way to set description in groups
      }

      rmp = (RMProperty) keywords.get("OWNER");
      if (!rmp.is_default) {
        try {
          User user = repository.getUser(rmp.value);
          group.setOwner(user);
        }
        catch (ReportMartException rme) {
          // error!!! perhaps update other fields...
          error("Error updating group \"" + name + "\", " +
                           rme.getMessage());
        }
      }
      rmp = (RMProperty) keywords.get("PERMISSIONS");
      if (!rmp.is_default) {
        group.setPermissions(convertUmask(rmp.value));
      }
      //RMLoader.logMsg("Group \"" + name + "\" successfully updated");
      warning("Warning: the current RM API does not support updating " +
                      "of group information.\n  Group \"" + name + "\" was " +
                      "not updated.");
      // group.update();
      printGoodRecord();
      // log("group \"" + name + "\" successfully updated");
    }
    catch (ReportMartException rme) {
      error("Error updating group: " + rme.getMessage());
      printBadRecord();
    }
  }

  /**
  *
  *  Permanently remove a group from ReportMart
  *
  **/
  private void deleteGroup() {
    RMProperty rmp = (RMProperty)keywords.get("NAME");
    String name = rmp.value;

    // check for ignored keywords
    Enumeration e = keywords.elements();
    boolean error_status = false;
    while (e.hasMoreElements()) {
      rmp = (RMProperty)e.nextElement();
      if (!rmp.name.equals("NAME") && !rmp.is_default) {
        switch(RMLoader.config.when_ignored_keyword) {
        case RMConfig.ERROR :
          error_status = true;
          error("ignored keyword, " +rmp.name + ", group will not be deleted");
          break;
        case RMConfig.WARNING :
          warning("ignored keyword, " + rmp.name);
        case RMConfig.IGNORE :
        }
      }
    }
    if (error_status) {
      printBadRecord();   
      return;
    }
    try {
      Group group = repository.getGroup(name);
      repository.removeGroup(group);
      log("Group \"" + name + "\" successfully deleted");
    }
    catch (ReportMartException rme) {
      // what to do? check config
      error("unable to delete group \"" + name + "\": " + rme.getMessage());
    }
  } // private deleteGroup()
} // public class GroupAction
